/* We don't want gtkdoc scanning this file, it'll get
 * confused by seeing function definitions, so mark it as: */
/*<private_header>*/

/**
 * emp_cli_channel_handler_callback_for_handle_channel:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a HandleChannel method call
 * succeeds or fails.
 */
static void
_emp_cli_channel_handler_collect_callback_handle_channel (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_channel_handler_invoke_callback_handle_channel (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_channel_handler_callback_for_handle_channel callback = (emp_cli_channel_handler_callback_for_handle_channel) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_channel_handler_call_handle_channel:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_Bus_Name: Used to pass an 'in' argument: The bus name of the connection and channel
 * @in_Connection: Used to pass an 'in' argument: The object-path of the connection that owns the channel
 * @in_Channel_Type: Used to pass an 'in' argument: The channel type
 * @in_Channel: Used to pass an 'in' argument: The object-path of the channel
 * @in_Handle_Type: Used to pass an 'in' argument: The type of the handle that the channel communicates           with, or 0 if there is no associated handle
 * @in_Handle: Used to pass an 'in' argument: The handle that the channel communicates with,           or 0 if there is no associated handle
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a HandleChannel method call.
 *
 * Called when a channel handler should handle a new channel.
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_channel_handler_call_handle_channel (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_Bus_Name,
    const gchar *in_Connection,
    const gchar *in_Channel_Type,
    const gchar *in_Channel,
    guint in_Handle_Type,
    guint in_Handle,
    emp_cli_channel_handler_callback_for_handle_channel callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_HANDLER;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "HandleChannel",
          G_TYPE_STRING, in_Bus_Name,
          DBUS_TYPE_G_OBJECT_PATH, in_Connection,
          G_TYPE_STRING, in_Channel_Type,
          DBUS_TYPE_G_OBJECT_PATH, in_Channel,
          G_TYPE_UINT, in_Handle_Type,
          G_TYPE_UINT, in_Handle,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "HandleChannel", iface,
          _emp_cli_channel_handler_invoke_callback_handle_channel,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "HandleChannel",
              _emp_cli_channel_handler_collect_callback_handle_channel,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_STRING, in_Bus_Name,
              DBUS_TYPE_G_OBJECT_PATH, in_Connection,
              G_TYPE_STRING, in_Channel_Type,
              DBUS_TYPE_G_OBJECT_PATH, in_Channel,
              G_TYPE_UINT, in_Handle_Type,
              G_TYPE_UINT, in_Handle,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _emp_cli_channel_handler_run_state_handle_channel;
static void
_emp_cli_channel_handler_finish_running_handle_channel (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_channel_handler_run_state_handle_channel *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_channel_handler_run_handle_channel:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_Bus_Name: Used to pass an 'in' argument: The bus name of the connection and channel
 * @in_Connection: Used to pass an 'in' argument: The object-path of the connection that owns the channel
 * @in_Channel_Type: Used to pass an 'in' argument: The channel type
 * @in_Channel: Used to pass an 'in' argument: The object-path of the channel
 * @in_Handle_Type: Used to pass an 'in' argument: The type of the handle that the channel communicates           with, or 0 if there is no associated handle
 * @in_Handle: Used to pass an 'in' argument: The handle that the channel communicates with,           or 0 if there is no associated handle
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method HandleChannel and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * Called when a channel handler should handle a new channel.
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_channel_handler_run_handle_channel (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_Bus_Name,
    const gchar *in_Connection,
    const gchar *in_Channel_Type,
    const gchar *in_Channel,
    guint in_Handle_Type,
    guint in_Handle,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_HANDLER;
  TpProxyPendingCall *pc;
  _emp_cli_channel_handler_run_state_handle_channel state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "HandleChannel", iface,
      _emp_cli_channel_handler_finish_running_handle_channel,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "HandleChannel",
          _emp_cli_channel_handler_collect_callback_handle_channel,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              G_TYPE_STRING, in_Bus_Name,
              DBUS_TYPE_G_OBJECT_PATH, in_Connection,
              G_TYPE_STRING, in_Channel_Type,
              DBUS_TYPE_G_OBJECT_PATH, in_Channel,
              G_TYPE_UINT, in_Handle_Type,
              G_TYPE_UINT, in_Handle,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


static inline void
emp_cli_add_signals_for_stream_engine (DBusGProxy *proxy)
{
  if (!tp_proxy_dbus_g_proxy_claim_for_signal_adding (proxy))
    return;
  dbus_g_proxy_add_signal (proxy, "Receiving",
      DBUS_TYPE_G_OBJECT_PATH,
      G_TYPE_UINT,
      G_TYPE_BOOLEAN,
      G_TYPE_INVALID);
  dbus_g_proxy_add_signal (proxy, "StreamStateChanged",
      DBUS_TYPE_G_OBJECT_PATH,
      G_TYPE_UINT,
      G_TYPE_UINT,
      G_TYPE_UINT,
      G_TYPE_INVALID);
}


/**
 * emp_cli_stream_engine_signal_callback_receiving:
 * @proxy: The proxy on which emp_cli_stream_engine_connect_to_receiving ()
 *  was called
 * @arg_channel_path: (Undocumented)
 * @arg_stream_id: (Undocumented)
 * @arg_state: (Undocumented)
 * @user_data: User-supplied data
 * @weak_object: User-supplied weakly referenced object
 *
 * Represents the signature of a callback for the signal Receiving.
 */
static void
_emp_cli_stream_engine_collect_args_of_receiving (DBusGProxy *proxy G_GNUC_UNUSED,
    const gchar *arg_channel_path,
    guint arg_stream_id,
    gboolean arg_state,
    TpProxySignalConnection *sc)
{
  GValueArray *args = g_value_array_new (3);
  GValue blank = { 0 };
  guint i;

  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 3; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, DBUS_TYPE_G_OBJECT_PATH);
  g_value_set_boxed (args->values + 0, arg_channel_path);

  g_value_unset (args->values + 1);
  g_value_init (args->values + 1, G_TYPE_UINT);
  g_value_set_uint (args->values + 1, arg_stream_id);

  g_value_unset (args->values + 2);
  g_value_init (args->values + 2, G_TYPE_BOOLEAN);
  g_value_set_boolean (args->values + 2, arg_state);

  tp_proxy_signal_connection_v0_take_results (sc, args);
}
static void
_emp_cli_stream_engine_invoke_callback_for_receiving (TpProxy *tpproxy,
    GError *error G_GNUC_UNUSED,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_stream_engine_signal_callback_receiving callback =
      (emp_cli_stream_engine_signal_callback_receiving) generic_callback;

  if (callback != NULL)
    callback (g_object_ref (tpproxy),
      g_value_get_boxed (args->values + 0),
      g_value_get_uint (args->values + 1),
      g_value_get_boolean (args->values + 2),
      user_data,
      weak_object);

  g_value_array_free (args);
  g_object_unref (tpproxy);
}
/**
 * emp_cli_stream_engine_connect_to_receiving:
 * @proxy: A #TpProxy or subclass
 * @callback: Callback to be called when the signal is
 *   received
 * @user_data: User-supplied data for the callback
 * @destroy: Destructor for the user-supplied data, which
 *   will be called when this signal is disconnected, or
 *   before this function returns %NULL
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 * @error: If not %NULL, used to raise an error if %NULL is
 *   returned
 *
 * Connect a handler to the signal Receiving.
 *
 * (Undocumented)
 *
 * Returns: a #TpProxySignalConnection containing all of the
 * above, which can be used to disconnect the signal; or
 * %NULL if the proxy does not have the desired interface
 * or has become invalid.
 */
TpProxySignalConnection *
emp_cli_stream_engine_connect_to_receiving (TpProxy *proxy,
    emp_cli_stream_engine_signal_callback_receiving callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object,
    GError **error)
{
  GType expected_types[4] = {
      DBUS_TYPE_G_OBJECT_PATH,
      G_TYPE_UINT,
      G_TYPE_BOOLEAN,
      G_TYPE_INVALID };

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL, NULL);

  return tp_proxy_signal_connection_v0_new ((TpProxy *) proxy,
      EMP_IFACE_QUARK_STREAM_ENGINE, "Receiving",
      expected_types,
      G_CALLBACK (_emp_cli_stream_engine_collect_args_of_receiving),
      _emp_cli_stream_engine_invoke_callback_for_receiving,
      G_CALLBACK (callback), user_data, destroy,
      weak_object, error);
}

/**
 * emp_cli_stream_engine_signal_callback_stream_state_changed:
 * @proxy: The proxy on which emp_cli_stream_engine_connect_to_stream_state_changed ()
 *  was called
 * @arg_channel_path: (Undocumented)
 * @arg_stream_id: (Undocumented)
 * @arg_state: (Undocumented)
 * @arg_direction: (Undocumented)
 * @user_data: User-supplied data
 * @weak_object: User-supplied weakly referenced object
 *
 * Represents the signature of a callback for the signal StreamStateChanged.
 */
static void
_emp_cli_stream_engine_collect_args_of_stream_state_changed (DBusGProxy *proxy G_GNUC_UNUSED,
    const gchar *arg_channel_path,
    guint arg_stream_id,
    guint arg_state,
    guint arg_direction,
    TpProxySignalConnection *sc)
{
  GValueArray *args = g_value_array_new (4);
  GValue blank = { 0 };
  guint i;

  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 4; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, DBUS_TYPE_G_OBJECT_PATH);
  g_value_set_boxed (args->values + 0, arg_channel_path);

  g_value_unset (args->values + 1);
  g_value_init (args->values + 1, G_TYPE_UINT);
  g_value_set_uint (args->values + 1, arg_stream_id);

  g_value_unset (args->values + 2);
  g_value_init (args->values + 2, G_TYPE_UINT);
  g_value_set_uint (args->values + 2, arg_state);

  g_value_unset (args->values + 3);
  g_value_init (args->values + 3, G_TYPE_UINT);
  g_value_set_uint (args->values + 3, arg_direction);

  tp_proxy_signal_connection_v0_take_results (sc, args);
}
static void
_emp_cli_stream_engine_invoke_callback_for_stream_state_changed (TpProxy *tpproxy,
    GError *error G_GNUC_UNUSED,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_stream_engine_signal_callback_stream_state_changed callback =
      (emp_cli_stream_engine_signal_callback_stream_state_changed) generic_callback;

  if (callback != NULL)
    callback (g_object_ref (tpproxy),
      g_value_get_boxed (args->values + 0),
      g_value_get_uint (args->values + 1),
      g_value_get_uint (args->values + 2),
      g_value_get_uint (args->values + 3),
      user_data,
      weak_object);

  g_value_array_free (args);
  g_object_unref (tpproxy);
}
/**
 * emp_cli_stream_engine_connect_to_stream_state_changed:
 * @proxy: A #TpProxy or subclass
 * @callback: Callback to be called when the signal is
 *   received
 * @user_data: User-supplied data for the callback
 * @destroy: Destructor for the user-supplied data, which
 *   will be called when this signal is disconnected, or
 *   before this function returns %NULL
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 * @error: If not %NULL, used to raise an error if %NULL is
 *   returned
 *
 * Connect a handler to the signal StreamStateChanged.
 *
 * (Undocumented)
 *
 * Returns: a #TpProxySignalConnection containing all of the
 * above, which can be used to disconnect the signal; or
 * %NULL if the proxy does not have the desired interface
 * or has become invalid.
 */
TpProxySignalConnection *
emp_cli_stream_engine_connect_to_stream_state_changed (TpProxy *proxy,
    emp_cli_stream_engine_signal_callback_stream_state_changed callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object,
    GError **error)
{
  GType expected_types[5] = {
      DBUS_TYPE_G_OBJECT_PATH,
      G_TYPE_UINT,
      G_TYPE_UINT,
      G_TYPE_UINT,
      G_TYPE_INVALID };

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL, NULL);

  return tp_proxy_signal_connection_v0_new ((TpProxy *) proxy,
      EMP_IFACE_QUARK_STREAM_ENGINE, "StreamStateChanged",
      expected_types,
      G_CALLBACK (_emp_cli_stream_engine_collect_args_of_stream_state_changed),
      _emp_cli_stream_engine_invoke_callback_for_stream_state_changed,
      G_CALLBACK (callback), user_data, destroy,
      weak_object, error);
}

/**
 * emp_cli_stream_engine_callback_for_set_output_volume:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a SetOutputVolume method call
 * succeeds or fails.
 */
static void
_emp_cli_stream_engine_collect_callback_set_output_volume (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_stream_engine_invoke_callback_set_output_volume (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_stream_engine_callback_for_set_output_volume callback = (emp_cli_stream_engine_callback_for_set_output_volume) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_call_set_output_volume:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_channel_path: Used to pass an 'in' argument: (Undocumented)
 * @in_stream_id: Used to pass an 'in' argument: (Undocumented)
 * @in_volume: Used to pass an 'in' argument: (Undocumented)
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a SetOutputVolume method call.
 *
 * (Undocumented)
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_stream_engine_call_set_output_volume (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_channel_path,
    guint in_stream_id,
    guint in_volume,
    emp_cli_stream_engine_callback_for_set_output_volume callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "SetOutputVolume",
          DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
          G_TYPE_UINT, in_stream_id,
          G_TYPE_UINT, in_volume,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "SetOutputVolume", iface,
          _emp_cli_stream_engine_invoke_callback_set_output_volume,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "SetOutputVolume",
              _emp_cli_stream_engine_collect_callback_set_output_volume,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
              G_TYPE_UINT, in_stream_id,
              G_TYPE_UINT, in_volume,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _emp_cli_stream_engine_run_state_set_output_volume;
static void
_emp_cli_stream_engine_finish_running_set_output_volume (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_stream_engine_run_state_set_output_volume *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_run_set_output_volume:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_channel_path: Used to pass an 'in' argument: (Undocumented)
 * @in_stream_id: Used to pass an 'in' argument: (Undocumented)
 * @in_volume: Used to pass an 'in' argument: (Undocumented)
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method SetOutputVolume and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * (Undocumented)
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_stream_engine_run_set_output_volume (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_channel_path,
    guint in_stream_id,
    guint in_volume,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  TpProxyPendingCall *pc;
  _emp_cli_stream_engine_run_state_set_output_volume state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "SetOutputVolume", iface,
      _emp_cli_stream_engine_finish_running_set_output_volume,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "SetOutputVolume",
          _emp_cli_stream_engine_collect_callback_set_output_volume,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
              G_TYPE_UINT, in_stream_id,
              G_TYPE_UINT, in_volume,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * emp_cli_stream_engine_callback_for_mute_input:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a MuteInput method call
 * succeeds or fails.
 */
static void
_emp_cli_stream_engine_collect_callback_mute_input (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_stream_engine_invoke_callback_mute_input (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_stream_engine_callback_for_mute_input callback = (emp_cli_stream_engine_callback_for_mute_input) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_call_mute_input:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_channel_path: Used to pass an 'in' argument: (Undocumented)
 * @in_stream_id: Used to pass an 'in' argument: (Undocumented)
 * @in_mute_state: Used to pass an 'in' argument: (Undocumented)
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a MuteInput method call.
 *
 * (Undocumented)
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_stream_engine_call_mute_input (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_channel_path,
    guint in_stream_id,
    gboolean in_mute_state,
    emp_cli_stream_engine_callback_for_mute_input callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "MuteInput",
          DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
          G_TYPE_UINT, in_stream_id,
          G_TYPE_BOOLEAN, in_mute_state,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "MuteInput", iface,
          _emp_cli_stream_engine_invoke_callback_mute_input,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "MuteInput",
              _emp_cli_stream_engine_collect_callback_mute_input,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
              G_TYPE_UINT, in_stream_id,
              G_TYPE_BOOLEAN, in_mute_state,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _emp_cli_stream_engine_run_state_mute_input;
static void
_emp_cli_stream_engine_finish_running_mute_input (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_stream_engine_run_state_mute_input *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_run_mute_input:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_channel_path: Used to pass an 'in' argument: (Undocumented)
 * @in_stream_id: Used to pass an 'in' argument: (Undocumented)
 * @in_mute_state: Used to pass an 'in' argument: (Undocumented)
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method MuteInput and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * (Undocumented)
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_stream_engine_run_mute_input (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_channel_path,
    guint in_stream_id,
    gboolean in_mute_state,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  TpProxyPendingCall *pc;
  _emp_cli_stream_engine_run_state_mute_input state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "MuteInput", iface,
      _emp_cli_stream_engine_finish_running_mute_input,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "MuteInput",
          _emp_cli_stream_engine_collect_callback_mute_input,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
              G_TYPE_UINT, in_stream_id,
              G_TYPE_BOOLEAN, in_mute_state,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * emp_cli_stream_engine_callback_for_mute_output:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a MuteOutput method call
 * succeeds or fails.
 */
static void
_emp_cli_stream_engine_collect_callback_mute_output (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_stream_engine_invoke_callback_mute_output (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_stream_engine_callback_for_mute_output callback = (emp_cli_stream_engine_callback_for_mute_output) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_call_mute_output:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_channel_path: Used to pass an 'in' argument: (Undocumented)
 * @in_stream_id: Used to pass an 'in' argument: (Undocumented)
 * @in_mute_state: Used to pass an 'in' argument: (Undocumented)
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a MuteOutput method call.
 *
 * (Undocumented)
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_stream_engine_call_mute_output (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_channel_path,
    guint in_stream_id,
    gboolean in_mute_state,
    emp_cli_stream_engine_callback_for_mute_output callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "MuteOutput",
          DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
          G_TYPE_UINT, in_stream_id,
          G_TYPE_BOOLEAN, in_mute_state,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "MuteOutput", iface,
          _emp_cli_stream_engine_invoke_callback_mute_output,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "MuteOutput",
              _emp_cli_stream_engine_collect_callback_mute_output,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
              G_TYPE_UINT, in_stream_id,
              G_TYPE_BOOLEAN, in_mute_state,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _emp_cli_stream_engine_run_state_mute_output;
static void
_emp_cli_stream_engine_finish_running_mute_output (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_stream_engine_run_state_mute_output *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_run_mute_output:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_channel_path: Used to pass an 'in' argument: (Undocumented)
 * @in_stream_id: Used to pass an 'in' argument: (Undocumented)
 * @in_mute_state: Used to pass an 'in' argument: (Undocumented)
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method MuteOutput and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * (Undocumented)
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_stream_engine_run_mute_output (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_channel_path,
    guint in_stream_id,
    gboolean in_mute_state,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  TpProxyPendingCall *pc;
  _emp_cli_stream_engine_run_state_mute_output state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "MuteOutput", iface,
      _emp_cli_stream_engine_finish_running_mute_output,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "MuteOutput",
          _emp_cli_stream_engine_collect_callback_mute_output,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
              G_TYPE_UINT, in_stream_id,
              G_TYPE_BOOLEAN, in_mute_state,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * emp_cli_stream_engine_callback_for_set_output_window:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a SetOutputWindow method call
 * succeeds or fails.
 */
static void
_emp_cli_stream_engine_collect_callback_set_output_window (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_stream_engine_invoke_callback_set_output_window (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_stream_engine_callback_for_set_output_window callback = (emp_cli_stream_engine_callback_for_set_output_window) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_call_set_output_window:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_channel_path: Used to pass an 'in' argument: (Undocumented)
 * @in_stream_id: Used to pass an 'in' argument: (Undocumented)
 * @in_window: Used to pass an 'in' argument: (Undocumented)
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a SetOutputWindow method call.
 *
 * (Undocumented)
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_stream_engine_call_set_output_window (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_channel_path,
    guint in_stream_id,
    guint in_window,
    emp_cli_stream_engine_callback_for_set_output_window callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "SetOutputWindow",
          DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
          G_TYPE_UINT, in_stream_id,
          G_TYPE_UINT, in_window,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "SetOutputWindow", iface,
          _emp_cli_stream_engine_invoke_callback_set_output_window,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "SetOutputWindow",
              _emp_cli_stream_engine_collect_callback_set_output_window,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
              G_TYPE_UINT, in_stream_id,
              G_TYPE_UINT, in_window,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _emp_cli_stream_engine_run_state_set_output_window;
static void
_emp_cli_stream_engine_finish_running_set_output_window (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_stream_engine_run_state_set_output_window *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_run_set_output_window:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_channel_path: Used to pass an 'in' argument: (Undocumented)
 * @in_stream_id: Used to pass an 'in' argument: (Undocumented)
 * @in_window: Used to pass an 'in' argument: (Undocumented)
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method SetOutputWindow and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * (Undocumented)
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_stream_engine_run_set_output_window (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_channel_path,
    guint in_stream_id,
    guint in_window,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  TpProxyPendingCall *pc;
  _emp_cli_stream_engine_run_state_set_output_window state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "SetOutputWindow", iface,
      _emp_cli_stream_engine_finish_running_set_output_window,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "SetOutputWindow",
          _emp_cli_stream_engine_collect_callback_set_output_window,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              DBUS_TYPE_G_OBJECT_PATH, in_channel_path,
              G_TYPE_UINT, in_stream_id,
              G_TYPE_UINT, in_window,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * emp_cli_stream_engine_callback_for_add_preview_window:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a AddPreviewWindow method call
 * succeeds or fails.
 */
static void
_emp_cli_stream_engine_collect_callback_add_preview_window (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_stream_engine_invoke_callback_add_preview_window (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_stream_engine_callback_for_add_preview_window callback = (emp_cli_stream_engine_callback_for_add_preview_window) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_call_add_preview_window:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_window: Used to pass an 'in' argument: (Undocumented)
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a AddPreviewWindow method call.
 *
 * (Undocumented)
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_stream_engine_call_add_preview_window (TpProxy *proxy,
    gint timeout_ms,
    guint in_window,
    emp_cli_stream_engine_callback_for_add_preview_window callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "AddPreviewWindow",
          G_TYPE_UINT, in_window,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "AddPreviewWindow", iface,
          _emp_cli_stream_engine_invoke_callback_add_preview_window,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "AddPreviewWindow",
              _emp_cli_stream_engine_collect_callback_add_preview_window,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_UINT, in_window,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _emp_cli_stream_engine_run_state_add_preview_window;
static void
_emp_cli_stream_engine_finish_running_add_preview_window (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_stream_engine_run_state_add_preview_window *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_run_add_preview_window:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_window: Used to pass an 'in' argument: (Undocumented)
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method AddPreviewWindow and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * (Undocumented)
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_stream_engine_run_add_preview_window (TpProxy *proxy,
    gint timeout_ms,
    guint in_window,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  TpProxyPendingCall *pc;
  _emp_cli_stream_engine_run_state_add_preview_window state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "AddPreviewWindow", iface,
      _emp_cli_stream_engine_finish_running_add_preview_window,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "AddPreviewWindow",
          _emp_cli_stream_engine_collect_callback_add_preview_window,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              G_TYPE_UINT, in_window,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * emp_cli_stream_engine_callback_for_remove_preview_window:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a RemovePreviewWindow method call
 * succeeds or fails.
 */
static void
_emp_cli_stream_engine_collect_callback_remove_preview_window (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_stream_engine_invoke_callback_remove_preview_window (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_stream_engine_callback_for_remove_preview_window callback = (emp_cli_stream_engine_callback_for_remove_preview_window) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_call_remove_preview_window:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_window: Used to pass an 'in' argument: (Undocumented)
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a RemovePreviewWindow method call.
 *
 * (Undocumented)
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_stream_engine_call_remove_preview_window (TpProxy *proxy,
    gint timeout_ms,
    guint in_window,
    emp_cli_stream_engine_callback_for_remove_preview_window callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "RemovePreviewWindow",
          G_TYPE_UINT, in_window,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "RemovePreviewWindow", iface,
          _emp_cli_stream_engine_invoke_callback_remove_preview_window,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "RemovePreviewWindow",
              _emp_cli_stream_engine_collect_callback_remove_preview_window,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_UINT, in_window,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _emp_cli_stream_engine_run_state_remove_preview_window;
static void
_emp_cli_stream_engine_finish_running_remove_preview_window (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_stream_engine_run_state_remove_preview_window *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_run_remove_preview_window:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_window: Used to pass an 'in' argument: (Undocumented)
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method RemovePreviewWindow and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * (Undocumented)
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_stream_engine_run_remove_preview_window (TpProxy *proxy,
    gint timeout_ms,
    guint in_window,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  TpProxyPendingCall *pc;
  _emp_cli_stream_engine_run_state_remove_preview_window state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "RemovePreviewWindow", iface,
      _emp_cli_stream_engine_finish_running_remove_preview_window,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "RemovePreviewWindow",
          _emp_cli_stream_engine_collect_callback_remove_preview_window,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              G_TYPE_UINT, in_window,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * emp_cli_stream_engine_callback_for_shutdown:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a Shutdown method call
 * succeeds or fails.
 */
static void
_emp_cli_stream_engine_collect_callback_shutdown (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_stream_engine_invoke_callback_shutdown (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_stream_engine_callback_for_shutdown callback = (emp_cli_stream_engine_callback_for_shutdown) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_call_shutdown:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a Shutdown method call.
 *
 * (Undocumented)
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_stream_engine_call_shutdown (TpProxy *proxy,
    gint timeout_ms,
    emp_cli_stream_engine_callback_for_shutdown callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "Shutdown",
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "Shutdown", iface,
          _emp_cli_stream_engine_invoke_callback_shutdown,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "Shutdown",
              _emp_cli_stream_engine_collect_callback_shutdown,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _emp_cli_stream_engine_run_state_shutdown;
static void
_emp_cli_stream_engine_finish_running_shutdown (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_stream_engine_run_state_shutdown *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_stream_engine_run_shutdown:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method Shutdown and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * (Undocumented)
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_stream_engine_run_shutdown (TpProxy *proxy,
    gint timeout_ms,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_STREAM_ENGINE;
  TpProxyPendingCall *pc;
  _emp_cli_stream_engine_run_state_shutdown state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "Shutdown", iface,
      _emp_cli_stream_engine_finish_running_shutdown,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "Shutdown",
          _emp_cli_stream_engine_collect_callback_shutdown,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * emp_cli_tube_handler_callback_for_handle_tube:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a HandleTube method call
 * succeeds or fails.
 */
static void
_emp_cli_tube_handler_collect_callback_handle_tube (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_tube_handler_invoke_callback_handle_tube (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_tube_handler_callback_for_handle_tube callback = (emp_cli_tube_handler_callback_for_handle_tube) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_tube_handler_call_handle_tube:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_Bus_Name: Used to pass an 'in' argument: The bus name of the connection and channel
 * @in_Connection: Used to pass an 'in' argument: The object-path of the connection that owns the channel
 * @in_Channel: Used to pass an 'in' argument: The object-path of the tubes channel
 * @in_Handle_Type: Used to pass an 'in' argument: The type of the handle that the channel communicates           with, or 0 if there is no associated handle
 * @in_Handle: Used to pass an 'in' argument: The handle that the channel communicates with,           or 0 if there is no associated handle
 * @in_id: Used to pass an 'in' argument: The ID of the new tube.
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a HandleTube method call.
 *
 * Called when a tube handler should handle a new tube within a tubes channel.
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_tube_handler_call_handle_tube (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_Bus_Name,
    const gchar *in_Connection,
    const gchar *in_Channel,
    guint in_Handle_Type,
    guint in_Handle,
    guint in_id,
    emp_cli_tube_handler_callback_for_handle_tube callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_TUBE_HANDLER;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "HandleTube",
          G_TYPE_STRING, in_Bus_Name,
          DBUS_TYPE_G_OBJECT_PATH, in_Connection,
          DBUS_TYPE_G_OBJECT_PATH, in_Channel,
          G_TYPE_UINT, in_Handle_Type,
          G_TYPE_UINT, in_Handle,
          G_TYPE_UINT, in_id,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "HandleTube", iface,
          _emp_cli_tube_handler_invoke_callback_handle_tube,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "HandleTube",
              _emp_cli_tube_handler_collect_callback_handle_tube,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_STRING, in_Bus_Name,
              DBUS_TYPE_G_OBJECT_PATH, in_Connection,
              DBUS_TYPE_G_OBJECT_PATH, in_Channel,
              G_TYPE_UINT, in_Handle_Type,
              G_TYPE_UINT, in_Handle,
              G_TYPE_UINT, in_id,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _emp_cli_tube_handler_run_state_handle_tube;
static void
_emp_cli_tube_handler_finish_running_handle_tube (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_tube_handler_run_state_handle_tube *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_tube_handler_run_handle_tube:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_Bus_Name: Used to pass an 'in' argument: The bus name of the connection and channel
 * @in_Connection: Used to pass an 'in' argument: The object-path of the connection that owns the channel
 * @in_Channel: Used to pass an 'in' argument: The object-path of the tubes channel
 * @in_Handle_Type: Used to pass an 'in' argument: The type of the handle that the channel communicates           with, or 0 if there is no associated handle
 * @in_Handle: Used to pass an 'in' argument: The handle that the channel communicates with,           or 0 if there is no associated handle
 * @in_id: Used to pass an 'in' argument: The ID of the new tube.
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method HandleTube and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * Called when a tube handler should handle a new tube within a tubes channel.
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_tube_handler_run_handle_tube (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_Bus_Name,
    const gchar *in_Connection,
    const gchar *in_Channel,
    guint in_Handle_Type,
    guint in_Handle,
    guint in_id,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_TUBE_HANDLER;
  TpProxyPendingCall *pc;
  _emp_cli_tube_handler_run_state_handle_tube state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "HandleTube", iface,
      _emp_cli_tube_handler_finish_running_handle_tube,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "HandleTube",
          _emp_cli_tube_handler_collect_callback_handle_tube,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              G_TYPE_STRING, in_Bus_Name,
              DBUS_TYPE_G_OBJECT_PATH, in_Connection,
              DBUS_TYPE_G_OBJECT_PATH, in_Channel,
              G_TYPE_UINT, in_Handle_Type,
              G_TYPE_UINT, in_Handle,
              G_TYPE_UINT, in_id,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/*
 * emp_cli_misc_add_signals:
 * @self: the #TpProxy
 * @quark: a quark whose string value is the interface
 *   name whose signals should be added
 * @proxy: the D-Bus proxy to which to add the signals
 * @unused: not used for anything
 *
 * Tell dbus-glib that @proxy has the signatures of all
 * signals on the given interface, if it's one we
 * support.
 *
 * This function should be used as a signal handler for
 * #TpProxy::interface-added.
 */
static void
emp_cli_misc_add_signals (TpProxy *self G_GNUC_UNUSED,
    guint quark,
    DBusGProxy *proxy,
    gpointer unused G_GNUC_UNUSED)
{
  if (quark == EMP_IFACE_QUARK_STREAM_ENGINE)
    emp_cli_add_signals_for_stream_engine (proxy);
}
